<?php

namespace HS\Install\System\Checks;

use Illuminate\Database\DatabaseManager;
use HS\Install\System\SystemCheckInterface;

class SqlServerVersion implements SystemCheckInterface
{
    /**
     * @var \Illuminate\Database\DatabaseManager
     */
    protected $db;

    /**
     * If system check is valid (OK).
     * @var bool
     */
    protected $valid;

    /**
     * Error.
     * @var string
     */
    protected $error;

    public function __construct(DatabaseManager $db)
    {
        $this->db = $db;
    }

    /**
     * This is required to pass.
     * @return bool
     */
    public function required()
    {
        return true;
    }

    /**
     * If we cannot connect to the DB
     * we get an exception here.
     * @return bool
     */
    public function valid()
    {
        // if we aren't using sqlsrv then just pretend it works.
        if (config('database.default') != 'sqlsrv') {
            return true;
        }

        if (is_bool($this->valid)) {
            return $this->valid;
        }

        try {
            $result = $this->db->select("SELECT SERVERPROPERTY('productversion') as version;");
            $result = $result[0];

            //check if we are at least sql server 2012
            $this->valid = (version_compare($result->version, '11.0.2100.60', '>='));
        } catch (\Exception $e) {
            $this->error = $e->getMessage();
            $this->valid = false;
        }

        if ($this->valid === false) {
            $this->error = 'HelpSpot Requires At Least SQL Server 2012';
        }

        return $this->valid;
    }

    /**
     * @return mixed
     */
    public function getError()
    {
        return $this->error;
    }
}
